package com.fluxloop.pinch

import com.fluxloop.pinch.sdk.Pinch
import com.fluxloop.pinch.common.DemographicProfile
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

class PinchPlugin: MethodCallHandler {
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "com.fluxloop.pinch/sdk")
      channel.setMethodCallHandler(PinchPlugin())
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    try {
      when (call.method) {
        "start" -> {
          Pinch.start()
          result.success(true)
        }
        "stop" -> {
          Pinch.stop()
          result.success(true)
        }
        "setMessagingId" -> {
          val messagingId: String? = call.argument("messagingId")
          Pinch.messagingId = messagingId
          result.success(true)
        }
        "addCustomEvent" -> {
          val type: String? = call.argument("type")
          val json: String? = call.argument("json")
          if (type == null || json == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            Pinch.addCustomEvent(type, json)
            result.success(true)
          }
        }
        "setMetadata" -> {
          val type: String? = call.argument("type")
          val json: String? = call.argument("json")
          if (type == null || json == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            Pinch.setMetadata(type, json)
            result.success(true)
          }
        }
        "sendDemographicProfile" -> {
          val birthYear: Int? = call.argument("birthYear")
          val gender: String? = call.argument("gender")
          if (birthYear == null || gender == null) {
            result.error("Received invalid data", call.method, null);
          } else {
            val finalGender = when(gender) {
              "male" -> DemographicProfile.Gender.MALE
              "female" -> DemographicProfile.Gender.FEMALE
              "unknown" -> DemographicProfile.Gender.UNKNOWN
              else -> DemographicProfile.Gender.UNKNOWN
            }
            Pinch.sendDemographicProfile(birthYear, finalGender)
            result.success(true)
          }
        }
        "getStarted" -> {
          result.success(Pinch.started)
        }
      }
    } catch (e: Exception) {
      result.error("Exception encountered", call.method, e)
    }
  }
}
